# ESP32 wristband prototyping board

Thank you for your interest in the ESP32 wristband prototyping board, a general-purpose compact and extensible board for multiple use cases, designed for wearable designs. This board is based on the popular ESP32 platform and offers various features:
 * Bluetooth Low-Energy (BLE) and Wi-Fi connectivity
 * Low-power stand-by features with timer or push-button wake-up
 * Vibration
 * Sound notifications
 * 4 RGB LEDs
 * Battery-powered, with integrated battery protection circuit charge circuit
 * micro-USB connector for battery charge
 * Easy programming through a generic serial interface
 * External module support through an I2C connector

The ESP32 can be programmed in various ways:
 * [Arduino ESP32 core](https://github.com/espressif/arduino-esp32)
 * [ESP32-IDF](https://esp-idf.readthedocs.io/en/latest/)
 * [microPython](https://micropython.org/)

For simplicity and ease of migration of the existing Arduino libraries, the first one is used in the examples in this guide. To use it just install the [Arduino IDE](https://www.arduino.cc/en/Main/Software) and the [Arduino ESP32 core](https://github.com/espressif/arduino-esp32), following the [official instructions](https://github.com/espressif/arduino-esp32/blob/master/docs/arduino-ide/boards_manager.md).

### Setup and serial

The serial interface is exposed on the J3 connector:
 * Pin 1 (squared): **GND**
 * Pin 2: ESP32's **RX**
 * Pin 3: ESP32's **TX**

Additionally, to provide automatic reset and ease the programming within the Arduino IDE, the auxiliary signals are available on the J4 connector:
 * Pin 1 (squared): **BOOT_MODE (GPIO0)**
 * Pin 2: **EN**

A standard 3.3V USB to serial converter can be used to interface this board with a PC, just make sure that:
 * the **GND** of the converter is connected with the **GND** of this board
 * the **TX** of the converter is connected with the **RX** of this board, and vice versa
 * the auxiliary signals are connected, as explained [here](https://github.com/espressif/esptool/wiki/ESP32-Boot-Mode-Selection#automatic-bootloader):
   * **RTS** of the converter with **EN**
   * **DTR** of the converter with **BOOT_MODE (GPIO0)**

**Note that the pins ARE NOT 5V tolerant, so make sure that the serial converter is working at 3.3V, otherwise the board could be damaged.**

Now go to Tools and select:
 * **Board**: ESP32 Dev Module
 * **Upload Speed**: 115200 (higher speeds can be used, but might cause issues)
 * **Flash Frequency**: 80 MHz
 * **Flash Mode**: QIO
 * **Flash Size**: 4MB (32Mb) *
 * **Partition Scheme**: Default
 * **Core Debug Level**: None
 * **PSRAM**: Disabled

\* The board has a 16MB (128Mb) flash module, so this value can be increased if needed. As of September 2018, 4MB is the maximum size supported by this platform in the stable release of arduino-esp32, however support for 16MB flash memories with FAT filesystem [has been added recently](https://github.com/espressif/arduino-esp32/commit/7206b2f39707ad600e8bdfd6d74e4957189932dd), which can be obtained by using future stable releases or [development releases](https://github.com/espressif/arduino-esp32/blob/master/docs/arduino-ide/boards_manager.md).

Once done, the ESP32 can be programmed by in the Arduino IDE with the usual "Run" button.

The serial output can be seen in the usual Arduino serial monitor (or in any other serial monitor, as long as the hardware flow control/handshake is disabled) and can be used in the standard Arduino ways:

```
void setup() {
    Serial.begin(115200);
}

void loop() {
    Serial.println("Hello world!");
    delay(1000);
}
```

### RGB LEDs

The four RGB LEDs are not connected directly to the ESP32, instead they are connected to a Texas Instruments TLC59711, a 12-channel PWM driver. This IC can be used in Arduino through an [Adafruit library](https://github.com/adafruit/Adafruit_TLC59711/), however this library is outdated and uses the primary SPI interface (HSPI on the ESP32), while in this board it is connected to the secondary SPI interface (VSPI). For this reason, an [updated library has been developed](https://github.com/polifactory-polimi/Adafruit_TLC59711), which should be used instead.

The LEDS have the **RED** and **GREEN** channels swapped, and from left (push button) to right (microUSB) have the indexes **3** - **2** - **0** - **1**. Each PWM channel is 16-bit, so the brightness of each color ranges from 0 (switched off) to 65535 (full brightness).

The LED controller can be shut off to reduce the power consumption during standy periods by driving the **LED_EN** pin (GPIO4) high (or leaving it floating). To turn on the TLC59711 it is necessary then to drive low the **LED_EN** pin.

```
#include <SPI.h>
#include <Adafruit_TLC59711.h>

#define LED_EN_PIN 4

SPIClass vspi(VSPI);                // Use the VSPI interface
Adafruit_TLC59711 leds(1, &vspi);   // 1 IC

void setup() {
    pinMode(LED_EN_PIN, OUTPUT);
    digitalWrite(LED_EN_PIN, LOW);  // Turn on the TLC59711
    delay(10);                      // Wait for it...

    // LEDs are *REALLY* bright, use setBrightness to reduce
    // the maximum brightness (range 0-255)
    leds.setBrightness(15);

    leds.begin();
    leds.write();

    Serial.begin(115200);
}

void loop() {  
    // Note that each setLED() call only sets the intensity locally, use write() to apply the modifications
    leds.setLED(3, 65535, 65535, 65535);    // Leftmost LED (index 3) -> white (full brightness)
    leds.setLED(2, 0, 65535, 0);            // Left LED of the 3-LEDs group (index 2) -> red (half brightness)
    leds.setLED(0, 65535, 0, 0);            // Central LED of the 3-LEDs group (index 0) -> green (full brightness)
    leds.setLED(1, 0, 65535, 65535);        // Right LED of the 3-LEDs group (index 1) -> purple (full brightness)
    leds.write();
    delay(1000);

    // Turn off all the LEDs
    for (unsigned int i = 0; i < 4; i++)
        leds.setLED(i, 0, 0, 0);
    leds.write();
    delay(1000);
}
```

It is also possible to do nice fading effects:

```
#include <SPI.h>
#include <Adafruit_TLC59711.h>

#define LED_EN_PIN 4

SPIClass vspi(VSPI);                // Use the VSPI interface
Adafruit_TLC59711 leds(1, &vspi);   // 1 IC

void setup() {
    pinMode(LED_EN_PIN, OUTPUT);
    digitalWrite(LED_EN_PIN, LOW);  // Turn on the TLC59711
    delay(10);                      // Wait for it...

    // LEDs are *REALLY* bright, use setBrightness to reduce
    // the maximum brightness (range 0-255)
    leds.setBrightness(15);

    leds.begin();
    leds.write();

    Serial.begin(115200);
}

void loop() {  
    // Fading UP
    for (int i = 0; i <= 655; i++) {
        for (int j = 0; j < 4; j++) {
            leds.setLED(j, i * 100, i * 100, i * 100);
        }
        leds.write();
        delay(10);
    }

    // Fading DOWN
    for (int i = 655; i >= 0; i--) {
        for (int j = 0; j < 4; j++) {
            leds.setLED(j, i * 100, i * 100, i * 100);
        }
        leds.write();
        delay(10);
    }
}
```

### Buzzer

The buzzer is connected to the **GPIO14** and can be used by driving it with a square wave (which is a PWM waveform with a 50% of duty-cycle). This kind of buzzer produces the maximum sound intensity at 2.7 kHz, but other frequencies can be used. On the ESP32, analogWrite() **doesn't work** at the moment, however the LEDC module (misleading name, it has nothing to do with LEDs) can be controlled through its functions:

```
#define BUZZER_PIN 14
#define BUZZER_PWM_CH 1

void setup() {
    ledcAttachPin(BUZZER_PIN, BUZZER_PWM_CH);
    ledcSetup(BUZZER_PWM_CH, 2700, 1);      //2.7 kHz 1-bit PWM: either 100% (OFF) or 50% (ON) duty cycle
}

void loop() {
    ledcWrite(BUZZER_PWM_CH, 0);            // ON
    delay(1000);

    ledcWrite(BUZZER_PWM_CH, 1);            // OFF
    delay(1000);
}
```

### Vibration

A vibration motor is connected to **GPIO32**; the vibration intensity can be controlled through PWM, using the LEDC module as with the buzzer.

```
#define VIBRATION_PIN 32
#define VIBRATION_PWM_CH 2

void setup() {
    ledcAttachPin(VIBRATION_PIN, VIBRATION_PWM_CH);
    ledcSetup(VIBRATION_PWM_CH, 10000, 8);      //10 kHz 8-bit PWM (256 intensity levels)
}

void loop() {
    ledcWrite(VIBRATION_PWM_CH, 0);            // OFF
    delay(1000);

    ledcWrite(VIBRATION_PWM_CH, 128);          // Half power
    delay(1000);

    ledcWrite(VIBRATION_PWM_CH, 255);          // Full power
    delay(1000);
}
```

### Power management

A single cell LiPo 3.7 V battery can be connected to the BT1 connector:
 * Pin 1 (squared): **+VBATT**
 * Pin 2: **GND**

Three signals are available:
 * VBUS (USB) voltage
 * VBATT (LiPo battery) voltage
 * CHG (if charging)

VBUS and VBATT are connected respectively to **GPIO35** and **GPIO33** pins through a couple of resistive voltage divider, so their voltages can be computed with the ADC. The full scale voltage of the ESP32's ADC is 1.1V, however it has also a configurable attenuator to increase the maximum voltage. The ADC is highly not-linear and requires calibration, as shown in following example. Note that the calibration values in this example has been determined experimentally, and are not guaranteed to be correct or stable across different boards: you should measure the calibration values for each specific board.

```
#define VBUS_ADC_PIN 35
#define VBATT_ADC_PIN 33

int adc_to_mv(int r1, int r2, uint32_t value) {

  // 10 bit ADC, 11db attenuation
  // 0.140 V --> 0
  // 2.450 V --> 710

  const int cal_val = 710;
  const int cal_voltage = 2450;
  const int zero_voltage = 140;

  const int adc_voltage = value * (cal_voltage - zero_voltage) / 710 + zero_voltage;

  return adc_voltage * (r1 + r2) / r2;
}

void setup() {
    analogReadResolution(10);
    analogSetAttenuation(ADC_11db);

    Serial.begin(115200);
}

void loop() {
    int vbus_voltage = adc_to_mv(36, 33, analogRead(VBUS_ADC_PIN));
    int vbatt_voltage = adc_to_mv(270, 330, analogRead(VBATT_ADC_PIN));

    Serial.print("VBUS: ");
    Serial.print(vbus_voltage);
    Serial.println(" mV");

    Serial.print("VBATT: ");
    Serial.print(vbatt_voltage);
    Serial.println(" mV");

    delay(1000);
}
```

The VBATT voltage can be used to obtain an approximate battery charge percentage.
If the VBUS voltage is much lower than 5 V it means that the USB is not connected (or powered). However, if only the presence of the USB is required (and not its exact voltage), a simple digitalRead() can be used, which is faster than analogRead():

```
#define VBUS_ADC_PIN 35

void setup() {
    pinMode(VBUS_ADC_PIN, INPUT);

    Serial.begin(115200);
}

void loop() {
    if (digitalRead(VBUS_ADC_PIN) == HIGH)
        Serial.println("USB is connected and powered");
    else
        Serial.println("USB is either disconnected or unpowered");

    delay(1000);
}
```

Moreover the battery charge status can be obtained by the CHG pin (**GPIO16**). This pin (when configured as _INPUT\_PULLUP_) goes low when the battery charge is in progress, while it goes high if the battery is not charging (either because it is full or because the USB is not connected); this, joined with the VBUS information, can be used to monitor the charge status:

```
#define VBUS_ADC_PIN 35
#define CHARGING_PIN 16

void setup() {
    pinMode(VBUS_ADC_PIN, INPUT);
    pinMode(CHARGING_PIN, INPUT_PULLUP);

    Serial.begin(115200);
}

void loop() {
    if (digitalRead(VBUS_ADC_PIN) == LOW)  // USB not connected
        Serial.println("Not charging, insert USB to charge battery");
    else {
        if (digitalRead(CHARGING_PIN) == LOW)
            Serial.println("Battery charge in progress...");
        else
            Serial.println("Charge completed!");
    }

    delay(1000);
}
```

Note that, when the battery voltage is under the 3 V, a separate voltage controller (TPS3839DBZ) shuts down all the circuit to protect the battery.

### Switch button and sleep

A NO switch button is connected to **GPIO25**: when pressed, **GPIO25** goes low, otherwise it stays high.

```
#define BUTTON_PIN 25

void setup() {
    pinMode(BUTTON_PIN, INPUT);

    Serial.begin(115200);
}

void loop() {
    if (digitalRead(BUTTON_PIN) == LOW)
        Serial.println("Button pressed");
    else
        Serial.println("Button not pressed");

    delay(1000);
}
```

This button can be used as a wake up source:

```
#define BUTTON_PIN 25
#define LED_EN_PIN 4

void setup() {
    pinMode(BUTTON_PIN, INPUT);

    pinMode(LED_EN, OUTPUT);
    digitalWrite(LED_EN, LOW);      // Turn on the TLC59711

    Serial.begin(115200);
    Serial.println("I'm on!");
}

void loop() {

    // Do stuff...

    Serial.println("Preparing sleep...");

    // Don't forget to turn off the TLC59711 before powering off the ESP32:
    // the TLC59711 consumes power even when the LEDs are off and may drain quickly your battery
    digitalWrite(LED_EN, HIGH);      // Turn off the TLC59711

    // Tell the ESP32 that, if during the sleep the BUTTON_PIN input goes LOW (if the button
    // is pressed), it should wake up
    esp_sleep_enable_ext0_wakeup(gpio_num_t(BUTTON_PIN), 0);

    Serial.println("Going to sleep in 1 second, press the button to wake me up!");
    delay(1000);
    esp_deep_sleep_start();

    // Anything here will **NEVER** be executed: after the wake up the program will start
    // again from the beginning
}
```

All the variables are lost after going to sleep, except for the ones saved inside the RTC module, see [here](https://github.com/espressif/arduino-esp32/blob/master/libraries/ESP32/examples/DeepSleep/ExternalWakeUp/ExternalWakeUp.ino) for an example.

Alternatively, the wake up can be initiated by a timer; in this example, the ESP32 will wake up 5 seconds after going to sleep:

```
#define LED_EN_PIN 4

void setup() {
    pinMode(LED_EN, OUTPUT);
    digitalWrite(LED_EN, LOW);      // Turn on the TLC59711

    Serial.begin(115200);
    Serial.println("I'm on!");
}

void loop() {

    // Do stuff...

    Serial.println("Preparing sleep...");

    // Don't forget to turn off the TLC59711 before powering off the ESP32:
    // the TLC59711 consumes power even when the LEDs are off and may drain quickly your battery
    digitalWrite(LED_EN, HIGH);      // Turn off the TLC59711

    // Tell the ESP32 to wake up 5 seconds after going to sleep
    esp_sleep_enable_timer_wakeup(5 * 1000 * 1000);    // In microseconds

    Serial.println("Going to sleep in 1 second, see you after 5 seconds!");
    delay(1000);
    esp_deep_sleep_start();

    // Anything here will **NEVER** be executed: after the wake up the program will start
    // again from the beginning
}
```

If the TLC59711 module is turned off, the power consumption during the sleep is very low, in the order of the tenths of uA (some months to completely drain the battery).

### External boards

Additional boards can be connected to the ESP32 through the I2C bus on the J2 connector:
 * Pin 1 (squared): GND
 * Pin 2: +3.3 V
 * Pin 3: SCL
 * Pin 4: SDA

### Wi-Fi and Bluetooth

No modification has been done on the Wi-Fi and Bluetooth parts of the ESP32, so any example and program working for any ESP32 board will also work on this platform. As a reference, many examples have been written for the [ESP32 Arduino Wi-Fi library](https://github.com/espressif/arduino-esp32/tree/master/libraries/WiFi), the [ESP32 Arduino BluetoothSerial library](https://github.com/espressif/arduino-esp32/tree/master/libraries/BluetoothSerial) and the [ESP32 Arduino Bluetooth Low-Energy (BLE) library](https://github.com/nkolban/ESP32_BLE_Arduino), which can be used as starting point for the development of custom software.

### Miscellaneous

The remaining peripherals are managed by the corresponding libraries, either the [Arduino](https://github.com/espressif/arduino-esp32/tree/master/libraries) or the [ESP-IDF ones](https://docs.espressif.com/projects/esp-idf/en/latest/).

### Functionality test example

A functionality test software is listed here, which tests every peripheral on the board and then puts it in standby.

```
#include <SPI.h>
#include <Adafruit_TLC59711.h>

#define LED_EN_PIN 4

#define BUZZER_PIN 14
#define BUZZER_PWM_CH 1

#define VIBRATION_PIN 32
#define VIBRATION_PWM_CH 2

#define BUTTON_PIN 25

#define VBUS_ADC_PIN 35
#define VBATT_ADC_PIN 33
#define CHARGING_PIN 16

SPIClass vspi(VSPI);                // Use the VSPI interface
Adafruit_TLC59711 leds(1, &vspi);   // 1 IC

int adc_to_mv(int r1, int r2, uint32_t value) {

  // 10 bit ADC, 11db attenuation
  // 0.140 V --> 0
  // 2.450 V --> 710

  const int cal_val = 710;
  const int cal_voltage = 2450;
  const int zero_voltage = 140;

  const int adc_voltage = value * (cal_voltage - zero_voltage) / 710 + zero_voltage;

  return adc_voltage * (r1 + r2) / r2;
}

void setup() {
    pinMode(LED_EN_PIN, OUTPUT);
    digitalWrite(LED_EN_PIN, LOW);  // Turn on the TLC59711
    delay(10);                      // Wait for it...

    leds.setBrightness(15);
    leds.begin();
    leds.write();

    ledcAttachPin(BUZZER_PIN, BUZZER_PWM_CH);
    ledcSetup(BUZZER_PWM_CH, 2700, 1);

    ledcAttachPin(VIBRATION_PIN, VIBRATION_PWM_CH);
    ledcSetup(VIBRATION_PWM_CH, 10000, 8);      //10 kHz 8-bit PWM (256 intensity levels)

    pinMode(BUTTON_PIN, INPUT);

    pinMode(VBUS_ADC_PIN, INPUT);
    pinMode(CHARGING_PIN, INPUT_PULLUP);

    analogReadResolution(10);
    analogSetAttenuation(ADC_11db);

    Serial.begin(115200);
}

void loop() {  

    Serial.println("LEDs RED");
    for (int i = 0; i < 4; i++) {
        leds.setLED(i, 65535, 0, 0);
    }
    leds.write();
    delay(1000);

    Serial.println("LEDs GREEN");
    for (int i = 0; i < 4; i++) {
        leds.setLED(i, 0, 65535, 0);
    }
    leds.write();
    delay(1000);

    Serial.println("LEDs BLUE");
    for (int i = 0; i < 4; i++) {
        leds.setLED(i, 0, 0, 65535);
    }
    leds.write();
    delay(1000);

    for (int led = 0; led < 4; led++) {
        Serial.printf("LED %d WHITE\n", led);

        for (int i = 0; i < 4; i++) {
            if (i == led)
                leds.setLED(i, 65535, 65535, 65535);
            else
                leds.setLED(i, 0, 0, 0);
        }

        leds.write();
        delay(1000);
    }

    Serial.println("Vibration ON");
    ledcWrite(VIBRATION_PWM_CH, 255);          // Full power
    delay(1000);

    Serial.println("Vibration OFF");
    ledcWrite(VIBRATION_PWM_CH, 0);            // OFF
    delay(1000);

    Serial.println("Buzzer ON");
    ledcWrite(BUZZER_PWM_CH, 1);            // ON
    delay(500);

    Serial.println("Buzzer OFF");
    ledcWrite(BUZZER_PWM_CH, 0);            // OFF
    delay(1000);

    for (int i = 0; i < 3; i++) {
        if (digitalRead(VBUS_ADC_PIN) == LOW)
            Serial.println("Not charging, insert USB to charge battery");
        else {
            if (digitalRead(CHARGING_PIN) == LOW)
                Serial.println("Battery charge in progress...");
            else
                Serial.println("Charge completed!");
        }

        Serial.printf("Battery voltage: %d mV\n", adc_to_mv(270, 330, analogRead(VBATT_ADC_PIN)));

        delay(1000);
    }

    Serial.println("Going to sleep, press the button to restart");
    digitalWrite(LED_EN_PIN, HIGH);      // Turn off the TLC59711
    esp_sleep_enable_ext0_wakeup(gpio_num_t(BUTTON_PIN), 0);

    Serial.flush();
    esp_deep_sleep_start(); // Everything below this line will **NEVER** be executed
}
```
